
#include <stdbool.h>
#include <string.h>
#include "app_button.h"
#include "boards.h"
#include "app_gpiote.h"
#include "app_timer.h"
#include "nrf_delay.h"


#define APP_TIMER_PRESCALER             0  // Value of the RTC1 PRESCALER register.
#define APP_TIMER_MAX_TIMERS            3  // Maximum number of simultaneously created timers. // NORDIC One for debounce, one for long press detect
#define APP_TIMER_OP_QUEUE_SIZE         2  // Size of timer operation queues. 
#define BUTTON_DEBOUNCE_DELAY		    50 // Delay from a GPIOTE event until a button is reported as pushed. 
#define APP_GPIOTE_MAX_USERS            1  // Maximum number of users of the GPIOTE handler. 

#define MS_TO_TICKS(ms)                 (ms*32768/1000)
static app_timer_id_t                   m_timer_3_sec_id;
static app_timer_id_t                   m_timer_5_sec_id;

bool is_advertising = false;

/*
 * Handler to be called when button is pushed.
 * param[in]   pin_no 			The pin number where the event is genereated
 * param[in]   button_action 	Is the button pushed or released
 */
static void button_handler(uint8_t pin_no, uint8_t button_action)
{
    if(button_action == APP_BUTTON_PUSH)
    {
        switch(pin_no)
        {			
            case BUTTON_1:				
                app_timer_start(m_timer_3_sec_id, MS_TO_TICKS(3000), NULL); // NORDIC: Start 3 second single shot timer
                app_timer_start(m_timer_5_sec_id, MS_TO_TICKS(5000), NULL); // NORDIC: Start 5 second single shot timer
                break;
            default:
                break;
        }
    }
    else if(button_action == APP_BUTTON_RELEASE)
    {
        // Stop all timers if button is released
        app_timer_stop(m_timer_3_sec_id);
        app_timer_stop(m_timer_5_sec_id);
    }
}

/**
 * Initialize the clock.
 */
void init_clock()
{
    NRF_CLOCK->LFCLKSRC            = (CLOCK_LFCLKSRC_SRC_Xtal << CLOCK_LFCLKSRC_SRC_Pos);
    NRF_CLOCK->EVENTS_LFCLKSTARTED = 0;
    NRF_CLOCK->TASKS_LFCLKSTART    = 1;
    while (NRF_CLOCK->EVENTS_LFCLKSTARTED == 0); // Wait for clock to start
}	

/**
 * Initialize all four LEDs on the nRF51 DK.
 */
void init_leds()
{
    nrf_gpio_range_cfg_input(BUTTON_1, BUTTON_2, NRF_GPIO_PIN_PULLUP);
    nrf_gpio_range_cfg_output(LED_1, LED_4); 
    nrf_gpio_pin_set(LED_1);
    nrf_gpio_pin_set(LED_2);
    nrf_gpio_pin_set(LED_3);
    nrf_gpio_pin_set(LED_4);
}	

/*
 * Handler to be called on single shot timer event.
 */
static void timer_handler_3_sec(void * p_context)
{
    if(!nrf_gpio_pin_read(BUTTON_1))
    {
        if(is_advertising == false)
        {
            // Start advertising
            is_advertising = true; // Start advertising if not advertising already
            app_timer_stop(m_timer_5_sec_id); // Stop 5 sec timer since it is not necessary
        }
    }
}

/*
 * Handler to be called on single shot timer event.
 */
static void timer_handler_5_sec(void * p_context)
{
    if(!nrf_gpio_pin_read(BUTTON_1))
    {
        if(is_advertising == true)
        {
            is_advertising = false; // Stop advertising if button is still pressed after 5 second
        }
    }
}


/**
 * Function for application main entry.
 */
int main(void)
{	
    init_leds();
    init_clock();

    uint32_t err_code;

    // Button configuration structure.
    static app_button_cfg_t p_button[] = {  {BUTTON_1, APP_BUTTON_ACTIVE_LOW, NRF_GPIO_PIN_PULLUP, button_handler},
                                            {BUTTON_2, APP_BUTTON_ACTIVE_LOW, NRF_GPIO_PIN_PULLUP, button_handler}};
        
    // Macro for initializing the application timer module.
    // It will handle dimensioning and allocation of the memory buffer required by the timer, making sure that the buffer is correctly aligned. It will also connect the timer module to the scheduler (if specified).
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_MAX_TIMERS, APP_TIMER_OP_QUEUE_SIZE, NULL);    
    app_timer_create(&m_timer_3_sec_id, APP_TIMER_MODE_SINGLE_SHOT, timer_handler_3_sec); // NORDIC Create single shot timer triggering interrupt handler timer_handler_3_sec once
    app_timer_create(&m_timer_5_sec_id, APP_TIMER_MODE_SINGLE_SHOT, timer_handler_5_sec); // NORDIC Create single shot timer triggering interrupt handler timer_handler_3_sec once
    
    // Initializing the buttons.
    err_code = app_button_init(p_button, sizeof(p_button) / sizeof(p_button[0]), BUTTON_DEBOUNCE_DELAY);
    APP_ERROR_CHECK(err_code);
                                            
    // Enabling the buttons.										
    err_code = app_button_enable();
    APP_ERROR_CHECK(err_code);

    while(true)
    {
        // Blink LED 1 to indicate advertising
        if(is_advertising == 1)
        {
            // Start advertising here
            nrf_gpio_pin_toggle(LED_1);
            nrf_delay_ms(100);
        }
        else
        {
            // Stop advertising here. 
            nrf_gpio_pin_set(LED_1);
        }
    }
}

